<?php

namespace Drupal\menu_block_settings_override\Plugin\Block;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\menu_block\Plugin\Block\MenuBlock;
use Drupal\menu_block_settings_override\Plugin\Field\FieldType\MenuBlockSettingsOverrideItem;

/**
 * Provides an extended Menu block with overridable settings.
 *
 * @Block(
 *   id = "menu_block",
 *   admin_label = @Translation("Menu block"),
 *   category = @Translation("Menus"),
 *   deriver = "Drupal\menu_block\Plugin\Derivative\MenuBlock",
 *   forms = {
 *     "settings_tray" = "\Drupal\system\Form\SystemMenuOffCanvasForm",
 *   },
 *   context = {
 *     "node" = @ContextDefinition("entity:node", label = @Translation("Node"))
 *   }
 * )
 */
class MenuBlockWithOverridableSettings extends MenuBlock {

  public function getPluginDefinition() {
    $definition = parent::getPluginDefinition();
    // override the provider so the module is not added to dependencies and config entities that use this plugin can be taken over by the menu_block module on uninstall.
    $definition['provider'] = 'menu_block';
    return $definition;
  }

  /**
   * @return bool|MenuBlockSettingsOverrideItem
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  protected function getOverriddenSettings() {
    $context_values = $this->getContextValues();
    if (!array_key_exists('node', $context_values) || empty($context_values['node'])) {
      return FALSE;
    }
    /** @var \Drupal\node\Entity\Node $node */
    $node = $this->getContextValue('node');
    $menu_block_fields = array_values(array_filter($node->getFields(), function (FieldItemListInterface $field) {
      return $field->getItemDefinition()->getFieldDefinition()->getType() == 'menu_block_settings_override';
    }));
    if (empty($menu_block_fields)) {
      return FALSE;
    }
    // Currently we only allow one field per entity type - so it's safe to take the first result.
    /** @var \Drupal\menu_block\Plugin\Block\MenuBlockSettingsOverrideFieldItemList $field_value */
    $field_value = $menu_block_fields[0];
    /** @var \Drupal\menu_block\Plugin\Block\MenuBlockSettingsOverrideItem $overridden_settings */
    return $field_value->getValueForMenuBlockId($this->getPluginId());
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $overridden_settings = $this->getOverriddenSettings();
    if ($overridden_settings) {
      $hidden = $overridden_settings->get('hidden')->getCastedValue();
      $status = $overridden_settings->get('status')->getCastedValue();
      if ($hidden) {
        return ['#access' => FALSE];
      }
      if ($status) {
        foreach ($overridden_settings->getOverridableFields() as $key) {
          $this->overrideIfApplicable($key, $overridden_settings, $this->configuration);
        }
      }
    }
    // Cached on a per-node level thanks to the node context definition: https://drupal.stackexchange.com/a/199601
    return parent::build();
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    $config = parent::getConfiguration();
    $overridden_settings = $this->getOverriddenSettings();
    if ($overridden_settings) {
      $hidden = $overridden_settings->get('hidden')->getCastedValue();
      $status = $overridden_settings->get('status')->getCastedValue();

      if ($hidden) {
        return $config;
      }
      if ($status) {
        foreach ($overridden_settings->getOverridableFields() as $key) {
          $this->overrideIfApplicable($key, $overridden_settings, $config);
        }
      }
    }
    return $config;
  }

  /**
   * @param string $key
   * @param \Drupal\menu_block\Plugin\Block\MenuBlockSettingsOverrideItem $overridden_settings
   * @param array $configuration
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function overrideIfApplicable($key, MenuBlockSettingsOverrideItem $overridden_settings, array &$configuration) {
    if ($overridden_settings->isApplicable($key)) {
      $configuration[$key] = $overridden_settings->get($key)->getCastedValue();
    }
  }

}
